@ ------------------------------------------- Ramsey2a.g -----------------------------------------------

  Solves the deterministic Ramsey Model (see Ramsey2.g for
  details) via iterations over the value function using linear interpolation
  between grid points.

  The result is compared to that from value function iteration without
  interpolation.

----------------------------------------------------------------------------------------------------- @

new; 

library pgraph, dge, user;

/* Parameters of the problem */
alpha=0.35;               @ elasticity of production with respect to capital @
beta=(1.065)^(-0.25);     @ discount factor                                  @
k0=0.1;                   @ initial value of capital stock                   @
eta=1;                    @ elasticity of marginal utility                   @
ngrid=50;                 @ number of grid points used in the value function iteration approach @

/* stationary solution */
ks=(alpha*beta)^(1/(1-alpha));  @ stationary capital stock                          @
cs=ks^alpha - ks;               @ stationary level of consumption                    @


grid=seqa(k0,(ks-k0)/(ngrid-2),ngrid);

v0=u(ks,ks)/(1-beta);
v0=ones(rows(grid),1).*v0;

bsec=hsec;
x1=SolveVIDc(&u,beta,grid,v0); @ the smarter method @
s1=hsec-bsec;

v1=zeros(rows(v0),1);
bsec=hsec;
x2=SolveVIDd; @ with linear interpolation @
s2=hsec-bsec;

?"Run Time: SolveVIDc: " etstr(s1);
?"Run Time: SolveDIDd: " etstr(s2);

/* Compute accuracy */
if eta==1;
   ktrue=alpha*beta*(grid^alpha);
   dk1=(grid[x1]-ktrue)./ktrue;
   dk2=(x2-ktrue)./ktrue;
   ?"Maximal Error without interpolation: " maxc(abs(dk1));
   ?"Maximal Error with    interpolation: " maxc(abs(dk2));
endif;

k0=0.1;

   
/* Compute time path for both solutions */
j=1; 
do until grid[j]>=k0;
 j=j+1;
endo;

kt1=grid[j];
do until x1[j]==x1[x1[j]];
   j=x1[j];
   kt1=kt1|grid[j];
endo;
  kt1=kt1|grid[x1[j]];

j=sumc(grid.<=k0);

kt2=grid[j];
k2=LinInter(grid,x2,grid[j]);
kt2=kt2|k2;
k1=grid[ngrid];
   do while k2/=k1;
      k1=k2;
      k2=LinInter(grid,x2,k1);
       kt2=kt2|k2;   
    endo;
   
n1=rows(kt1);
n2=rows(kt2);
if n2>n1;
   n3=n2;
   do while rows(kt1)<n2;
      kt1=kt1|kt1[n1];
   endo;
else;
   n3=n1;
   do while rows(kt2)<n1;
      kt2=kt2|kt2[n2];
   endo;
endif;

/* Compute time path from optimal solution it eta==1 */
if eta==1;
   kt3=zeros(n3,1);
   kt3[1]=k0;
   i=1;
   do until i>(n3-1);
      kt3[i+1]=alpha*beta*(kt3[i]^alpha);
             i=i+1;
   endo;
endif;

/* Plot deviations from true solution */
MyGraphSettings(0);
_plwidth=5;
_pltype={6,3};
@title("Deviation of the Policy Function from the Exact Solution");@
xlabel("Grid Points");
ylabel("Percent");
_plegctl=(1~4~grid[1]~100*maxc(maxc(dk1~dk2)));
_plegstr="Without\000With Linear Interpolation";
xy(grid,100*dk1~100*dk2);
wait;
/* write output to file */
output file="Ramsey2a.txt" reset;
?grid~(100*dk1)~(100*dk2);
output off;

goto ende;
/* Plot time paths */
MyGraphSettings(0);
_plwidth=7;
if eta==1;
   title("Deviations from Exact Time Path");
   _plegctl=(1~4~1~maxc(kt1-kt3));
   _plegstr="Without\000With Linear Interpolation";
   xy(seqa(1,1,n3),abs(kt1-kt3)~abs(kt2-kt3));
else;
   xy(seqa(1,1,n3),kt1~kt2);
endif;
  
ende:
end;

/* Definition of the utility function u=c^(1-eta)/(1-eta) as
** a function of this and next period's capital stock k1 and k2
** respectively */

proc(1)=u(k1,k2);

  local c;
  c=k1^alpha-k2;
  if c<0;
    c=miss(1,1);  @ set c to Gauss missing value code if c<0 @
  else;
    if eta==1;
      c=ln(c);
    else;  
      c=(c^(1-eta))/(1-eta);
    endif;
  endif;
 retp(c);
 
endp;
  
  
@---------------------------------- SolveVDd ----------------------------------

   Purpose: Computes the policy function for a simple deterministic, infinite
            horizon Ramsey model. In this model, capital K is the single
            (variable) factor of production and depreciates at the constant
            rate delta (must be in the interval (0,1]). The production function
            f(K) is strictly increasing in K, the utility function u(C)
            is strictly concave in C=f(K)+(1-delta)*K-K'.
             
   Algorithm: The policy function is computed on a grid of n points,
              supplied by the user in the column vector grid. The
              method employed is iteration over the value function.
                
              The value function is interpolated linearly between grid points.

              The iterations exploit the fact that the policy 
              function is monotonically increasing in the
              capital stock.
              
              The maximization steps use the concavity of the
              maximand.

   Usage:  k=SolveVIDd;

  Input :  Instead of passing &rf, beta, grid and v0
           to this procedure, you must use and initialize the following global
           symbols in your program before calling SolveVIDd:

           u(k1,k2): the utility function, note: it MUST HAVE THIS NAME

           beta    : the discount factor

           grid    : the n times 1 grid for the stock of capital

           v0      : the n times 1 vector with the initial value function

           v1      : the n times 1 vector that stores the final value function


  Output:  k  := n times 1 column vector of capital stocks.
                 Together with grid this solution can be used
                 to approximate the policy function by linear interpolation:
                 if grid[i]<=k1<=grid[i+1] the approximate best choice
                 k1'=g(k1) is given by 

                  k1'= k[i]+((k[i+1]-k[i])/(grid[i+1]-grid[i]))*(k[i]-grid[i])


------------------------------------------------------------------------------------ @

  
proc(1)=SolveVIDd;

   local i, jmin, jmax, jl, ju, js, w0, w, v2, k, k1, k2, t,
         Tmax, dv, eps1, neg, eps, ngrid, ok;    

    Tmax=1000;          @ maximum number of iterations @
    neg=-1e10;
    eps=0.01;    
    ngrid=rows(grid);

    /* Step 1: Initialize */
    eps1=0.01*(1-beta);     @ convergence criteria @

    k=grid;           
    k1=k; @ intial policy function @
    k2=zeros(ngrid,1);  


    v1=v0;  @ initial value function @
    v2=v1;
    dv=1;
    
    /* Step 2: Iterate over the value function */
    cls;    
    t=1;
    do until (t>Tmax) or ((dv<eps1) and (t>10));

        i=1;js=0;
        do until i>ngrid;   
            ok=varput(k[i],"k0"); @ make k[i] accessible to the function value1 @
            jmin=maxc(0|js-2);
            w0=neg;
            w=w0+1;
            /* step 3.1. find the optimal j^* */
            jl=jmin;
            do until (jl==ngrid) or (w<=w0);
                jl=jl+1;
                w0=w;
                w=u(k[i],k[jl])+beta*v1[jl];
                js=jl-1;
            endo;
            
    

    /* step 3.2. find optimal K'  with linear interpolation */       

            if js==1;  /* boundary optimum, ax=bx=a[1]  */
                if bellman(k[i],k[1]*eps*(k[2]-k[1]))<bellman(k[i],k[1]);
                    k2[i]=k[1];
                else;                  
                     k2[i]=GSS(&value1,k[1],k[2]);
                endif;
            elseif js==ngrid;    /* boundary optimum, bx=cx=a[n] */
                
                if bellman(k[i],k[ngrid]-eps*(k[ngrid]-k[ngrid-1]))<bellman(k[i],k[ngrid]);
                    k2[i]=k[ngrid];
                else;                
                      k2[i]=GSS(&value1,k[ngrid-1],k[ngrid]);
                endif;
            else;                
                 k2[i]=GSS(&value1,k[js-1],k[js+1]);
            endif;
           
            v2[i]=bellman(k[i],k2[i]);
            i=i+1;
        endo;

    
        dv=maxc(maxc(abs(v2-v1))); 
        locate 5,5;
        ?"Iteration #= " ftos(t,"*.*lf",6,0);
        locate 6,5;
        ?"Largest element in v1-v0= " dv;

        v1=v2;
        k1=k2;
        t=t+1;
   
    endo;

    if t>tmax;
        ?"Maximum number of iterations exceeded. Change Tmax in SolveVIDc";
        ?"The computed solution may be inaccurate.";
        ?"Press any key...";wait;
    endif;
    if minc(k1)==k[1]; locate 9,5; ?"Policy function hits lower bound of grid"; endif;
    if maxc(k1)==k[ngrid]; locate 10,5;?"Policy function hits upper bound of grid"; endif;

    retp(k1);

endp;

@ ------------------------------ Bellman ---------------------------------------

  Purpose: Used by SolveVIDd. The procedure returns the rhs of
           the Bellman equation. It assumes that the following
           global variables exist: beta, grid, and v1;

-------------------------------------------------------------------------------- @

proc bellman(a0,a1);
   
   retp(u(a0,a1)+beta*LinInter(grid,v1,a1));

endp;

@ ------------------------------- Value1---------------------------------------

   Purpose: Used by SolveVIDd. The procedure defines the function
            rhs of the Bellman equation as a function of next period's
            capital stock.

------------------------------------------------------------------------------ @

proc value1(x);
    local k0;
    k0=varget("k0");
    retp(bellman(k0,x));
endp;

