@ ------------------------------- Ramsey4a.g -----------------------------------

  Date:    08/08/2002
  Author:  Alfred Maussner
  Purpose: Solve the stochastic infinite horizon Ramsey model with
  
             1. one-period utility function u(c)=(c^(1-eta)(1-N)^(theta*(1-eta)))/(1-eta) or
                         u(C)=ln(c) + theta*ln(1-N) for eta=1

             2. technology Y_t = Z_t (A_tK_t)^alpha K_t^(1-alpha)

             3. transition equations

                         K_{t+1} = Y_t + (1-delta)K_t - C_t
                         
                         Z_{t+1} = (Z_t^rho)*exp(epsilon_t), epsilon_t draw from a
                         normal distribution with mean 0 and standard deviation sigma

            Solution technique is deterministic extended path.
                    
  Hint: When you run this file under Gauss 4.0 open the
        Dos combatibility window to ensure that the programs
        messages during run time are correctly displayed
        (if you use the standard output window instead, for
         reasons unknow to us, you get a mess)        
----------------------------------------------------------------------------- @

/* clear memory */
new;

/* libraries */
library user, dge, pgraph;

/* parameters */
alpha=0.73;         @ elasticity of output with respect to labor    @
eta=2.0;            @ elasticity of maringal utility of consumpiton @
delta=0.011;        @ rate of capital depreciation                  @
a=1.005;            @ growth rate of output,4.5 percent per annum   @
nstar=0.13;         @ on third of time endowment devoted to work    @
rho=0.90;           @ autoregressive parameter of productivity shock@
sigma=0.0072;       @ standard deviation of epsilon                 @
rstar=1.0206;       @ gross quarterly return on stocks (DAX 30)     @
beta=(a^eta)/rstar; @ discount factor                               @
if beta>1; ?"Beta > 1"; ?"Programm stops"; goto ende; endif;
nobs=60;            @ Simulate the model for nobs periods           @

/* Stationary solution */
yk=((a^eta)-beta*(1-delta))/(beta*(1-alpha));   @ output-capital ratio      @
ck=yk+(1-delta-a);                              @ consumption-capital ratio @
yc=yk/ck;                                       @ inverse of average propensity to consume @
theta=(alpha*yc*(1-nstar))/nstar;               @ theta implied by nstar          @
kn=yk^(-1/alpha);                               @ capital-labor ratio             @
kstar=kn*nstar;                                 @ stationary capital stock k=K/A  @
istar=(a+delta-1)*kstar;                        @ stationary level of investment  @
cstar=yk*kstar - istar;                         @ stationary level of consumption @
kmax=delta^(-1/alpha);                          @ maximum sustainable capital stock at z=1 and n=1 @

@goto next1; @ @ comment out if you want to compute the impulse response @

/* This part of the program computes the response of
** hours, conumption and the stock of capital to a one-time productivity shock. */

bsec=hsec;

/* Solve for the time path of capital */
T=250;                              @ number of periods to converge @
zt=zeros(t,1);
zt[1]=exp(sigma*rndn(1,1));         @ Draw a one-time shock from the standard normal @
zt[1]=exp(sigma);
i=1;
do until i>(t-1);                   @ Compute the associated time path of the productivity shock @
     zt[i+1]=zt[i]^rho;
           i=i+1;
endo;

k1=ones(T-1,1).*kstar;
n1=ones(t,1).*nstar;
x1=k1|n1;                                    @ intitial values    @
k0=kstar;                                    @ set initial value of k equal to the stationary solution @
bounds=( ones(rows(x1),1).*(10*macheps))~
       ((ones(T-1,1).*kmax)|(ones(T,1).*0.9999));
cls;
output  file=Ramsey4aTest5.txt reset;
{fx,x2}=FixVMN(x1,bounds,&Sys1);             @ call to the routine that solve the Euler equations @
output off;
@goto next2;@
Graphsettings;
begwind;
window(2,1,0);
xy(seqa(1,1,T),(k0|x2[1:T-1])~ones(T,1).*kstar);
nextwind;
xy(seqa(1,1,T),x2[T:rows(x2)]~ones(T,1).*nstar);
endwind;
goto ende;
next2:
/* Iterate over T: try a longer time path and check whether
** there is a noticable change in the first period values of n and k
** between the shorter and the longer path */
again:
 
k1=x2[1:t-1];         @ save old solution @
n1=x2[t:rows(x2)];
kfirst=k1[1];
nfirst=n1[1];   
told=t;
t=t+5;                  @ increment t in steps of 5 additional periods @
k1=k1|ones(5,1).*kstar;
n1=n1|ones(5,1).*nstar;
x1=k1|n1;
bounds=(ones(rows(x1),1).*(10*macheps))~
        ((ones(T-1,1).*kmax)|(ones(T,1).*0.9999));
i=1;
do until i>5;
   zt=zt|(zt[rows(zt)]^rho);
    i=i+1;
endo;

{fx,x2}=FixVMN(x1,bounds,&Sys1,&Sys2);

ksecond=x2[1];
nsecond=x2[t];

krit=maxc(abs((kfirst|nfirst)-(ksecond|nsecond)));
locate 8,2;
?"krit= " krit; wait;

if krit>1e-6;
   goto again;
endif;

s2=hsec-bsec;

next1:

/* This part of the program simulates the model for a number of periods,
** t is set to 200, which has been found to give sufficient accuracy, even
** with large shocks */

bsec=hsec;
t=100;
eps=exp(sigma*rndn(nobs,1));  @ Derive the sequence of productivity shocks @

zshock=zeros(nobs,1);
zshock[1]=eps[1];
i=1;
do until i>(nobs-1);
  zshock[i+1]=(zshock[i]^rho)*eps[i+1];
            i=i+1;
endo;

kt=zeros(nobs+1,1);kt[1]=kstar;  @ Start at stationary equilibrium @
it=zeros(nobs,1);
ct=zeros(nobs,1);       
nt=zeros(nobs,1); 
zbar=maxc(zshock);
kbar=(zbar/delta)^(1/(1-alpha));
k1=ones(T-1,1).*kstar;
n1=ones(t,1).*nstar;
x1=k1|n1;                                    @ intitial values    @
k0=kstar;                                    @ set initial value of k equal to the stationary solution @
bounds=(ones(rows(x1),1).*(10*macheps))~
       ((ones(T-1,1).*kbar)|(ones(T,1).*0.9999));

itn=1;cls;

do while itn<=nobs;
   locate 1,2;?"Period #= " ftos(itn,"*.*lf",4,0);
  k0=kt[itn];                      @ set initial value of k equal to the previous stock of capital @
   zt=zeros(T,1);
  zt[1]=zshock[itn];               @ compute the expected time path of productivity shock from the current shock onward @
  i=1;
  do until i>(t-1);
     zt[i+1]=zt[i]^rho;
           i=i+1;
  endo;
  {fx,x2}=FixVMN(x1,bounds,&Sys1,&Sys2);
    kt[itn+1]=x2[1];
    it[itn]  =a*kt[itn+1]+(delta-1)*kt[itn];
    nt[itn]  =x2[t];
    ct[itn]  =zshock[itn]*(nt[itn]^alpha)*(kt[itn]^(1-alpha)) + (1-delta)*kt[itn] - a*kt[itn+1];
           x1=x2;        @ use old solution as guess for the next one @

itn=itn+1;

endo;
s3=hsec-bsec;

/* Plott time paths */
GraphSettings;
begwind;
window(2,2,0);
xlabel("Time");
xtics(1,nobs,1,0);
ylabel("Productivity Shock");
_plwidth=7;
_paxht=0.35;
xy(seqa(1,1,nobs),ZShock~ones(nobs,1));
nextwind;
ylabel("Investment");
xy(seqa(1,1,rows(it)),it~ones(nobs,1).*istar);
nextwind;
ylabel("Consumption");
xy(seqa(1,1,nobs),ct~ones(nobs,1).*cstar);
nextwind;
ylabel("Hours");
xy(seqa(1,1,nobs),nt~ones(nobs,1).*nstar);
endwind;

?"Runtime= " etstr(s3);
ende:
end;

/* ------------------------------------------ Subroutines ------------------------------ */

proc(1)=Sys1(x0);   @ defines the system of Euler equations for t=1 to T @

  local kt,nt,c1,c2,i,j,fx;
  
   kt=x0[1:T-1];
   nt=x0[T:rows(x0)];
   fx=zeros(rows(x0),1);
   
   c1=zt[1]*(nt[1]^alpha)*(k0^(1-alpha)) + (1-delta)*k0 - a*kt[1];
   c2=zt[2]*(nt[2]^alpha)*(kt[1]^(1-alpha)) + (1-delta)*kt[1] - a*kt[2];   
   fx[1]= theta*c1 - alpha*(1-nt[1])*zt[1]*(nt[1]^(alpha-1))*(k0^(1-alpha));
   fx[2]=((c2/c1)^eta)*(((1-nt[1])/(1-nt[2]))^(theta*(1-eta)))-
         beta*(a^(-eta))*(1-delta+(1-alpha)*zt[2]*(nt[2]^alpha)*(kt[1]^(-alpha)));
   
   i=2;j=3;
   do until i>(t-2);
         c1=c2;
         c2=zt[i+1]*(nt[i+1]^alpha)*(kt[i]^(1-alpha)) + (1-delta)*kt[i] - a*kt[i+1];      
      fx[j]=theta*c1-alpha*(1-nt[i])*zt[i]*(nt[i]^(alpha-1))*(kt[i-1]^(1-alpha));
    fx[j+1]=((c2/c1)^eta)*(((1-nt[i])/(1-nt[i+1]))^(theta*(1-eta)))
            -beta*(a^(-eta))*(1-delta+(1-alpha)*zt[i+1]*(nt[i+1]^alpha)*(kt[i]^(-alpha)));
      i=i+1;
      j=j+2;
   endo;         
         c1=c2;
         c2=zt[i+1]*(nt[i+1]^alpha)*(kt[i]^(1-alpha)) + (1-delta)*kt[i] - a*kstar;      
      fx[j]=theta*c1-alpha*(1-nt[i])*zt[i]*(nt[i]^(alpha-1))*(kt[i-1]^(1-alpha));
    fx[j+1]=((c2/c1)^eta)*(((1-nt[i])/(1-nt[i+1]))^(theta*(1-eta)))
            -beta*(a^(-eta))*(1-delta+(1-alpha)*zt[i+1]*(nt[i+1]^alpha)*(kt[i]^(-alpha))); 
    fx[j+2]=theta*c2-alpha*(1-nt[i+1])*zt[i+1]*(nt[i+1]^(alpha-1))*(kt[i]^(1-alpha));
     
retp(fx); 

endp;       
  
 
proc(1)=Sys2(x);

   local fx;
   external proc Sys1;
   fx=Sys1(x);
   retp((1/2)*(fx'fx));

endp;


proc(0)=wipe(rpos,cpos);
  local i; 
  locate rpos,cpos;
  ?"                                                                           ";
  i=1;
  do while i<20;
     locate rpos+i,cpos;
     ?"                                                                        ";
     i=i+1;
  endo;

retp;
endp;
  
