@ --------------------------- Ramsey2.g ------------------------------------------

  Date:   08/02/2002
  Author: Alfred Maussner
  Purpose: Solve the simple infinite horizon Ramsey model from
           example 1.2.1

           The program is able to deal with more general
           one-period utility function

           u(c)=(C^(1-eta)-1)/(1-eta), eta >0,

           whereas the example uses eta=1.

  Hint: When you run this file under Gauss 4.0 open the
        Dos combatibility window to ensure that the programs
        messages during run time are correctly displayed
        (if you use the standard output window instead for
         reasons unknow to us, you get a mess)
---------------------------------------------------------------------------------- @

new; @ clear memory @

library pgraph, dge, user;

/* Parameters of the problem */
alpha=0.35;               @ elasticity of production with respect to capital @
beta=(1.065)^(-0.25);     @ dicont factor                                    @
k0=0.1;                   @ intial value of capital stock                    @
eta=1;                    @ elasticity of marginal utility                   @
T=30;                     @ Time considered necessary to approach the stationary equilibrium @
ngrid=500;                @ number of grid points used in the value function iteration approach @

/* stationary solution */
ks=(alpha*beta)^(1/(1-alpha));  @ stationary capital stock                          @
cs=ks^alpha - ks;               @ stationary leve of consumption                    @

/* This section provides the code for computing the optimal path from
** backward iteration.
*/

/* Eigenvalues and eigenvectors at the stationary solution */
bsec=hsec;
j=gradp(&Sys1,(ks|cs));
{lambda,v}=Eigv(j);
i=sortind(lambda);
vs=v[.,i];
dc=vs[2,1]./vs[1,1];
dk=0.000001*ks;  @ pertubation of equilibrium capital stock @
k1=ks-dk;
c1=cs-dc*dk;

xt=zeros(2,t);
xt[1,1]=k1;
xt[2,1]=c1;

i=1;
do while (i<t and xt[1,i]>k0);
   xt[.,i+1]=Sys2(xt[.,i]);
           i=i+1;
endo;
s1=hsec-bsec;

kt1=rev(xt[1,1:i]');

/* The next section provides the code for solving the system of
** first order condition from i=0 through T with K[T]=ks by means
** of the modified Newton-Raphson method
*/
bsec=hsec;
x0=ks*ones(T-1,1);                @ intitial values    @
k0=kt1[1];                        @ set initial value of k0 equal to kt1[1] @
bounds=zeros(t-1,1)~ones(t-1,1);  @ bounds for the modified Newton-Raphson method @
{fx,x1}=FixvMN(x0,bounds,&Sys3);  @ solve the system of T equations @

/* Iterate over T: try a longer time path and check whether
** there is a noticable change between the shorter and the longer path */
x2=x1;

again:

t=t+5;
x0=x2|ones(5,1).*ks;bounds=bounds|bounds[1:5,.];
{fx,x1}=FixvMN(x0,bounds,&Sys3);

krit=sqrt((x2-x1[1:rows(x2)])'(x2-x1[1:rows(x2)]));

if krit>1e-6;
   x2=x1;
   goto again;
endif;
s2=hsec-bsec;

kt2=k0|x1[1:rows(x2)];

kt1=kt1[1:rows(kt1)]|ones(rows(kt2)-rows(kt1),1).*ks; @ since kt1 is shorter than kt2, make them equal length @

/* The next section provides the code to solve the problem by
** value function iteration over a grid of ngrid points equally
** spaced in the intervall [0, 1.1*ks]
*/

bsec=hsec;
grid=seqa(kt1[1],(ks-kt1[1])/(ngrid-2),ngrid);
x3=SolveVIDa(&u,beta,grid); @ the naive method @
s3=hsec-bsec;

bsec=hsec;
v0=u(ks,ks)/(1-beta);
v0=ones(rows(grid),1).*v0;
x4=SolveVIDc(&u,beta,grid,v0); @ the smarter method @
s4=hsec-bsec;

j=1;   @ construct the optimal path from k0 onwards @
do until grid[j]>=kt1[1];
 j=j+1;
endo;

kt3=grid[j];
do until x3[j]==x3[x3[j]];
   j=x3[j];
   kt3=kt3|grid[j];
endo;
  kt3=kt3|grid[x3[j]];
do while rows(kt3) < rows(kt1);
   kt3=kt3|grid[x3[j]];
endo;   

/* Compute the optimal path from the exact policy function, if eta==1 */
if eta==1;
   tend=rows(kt1);
   kt4=zeros(tend,1);
   i=1;
   kt4[1]=kt1[1];
   do until i>(tend-1);
      kt4[i+1]=alpha*beta*kt4[i]^alpha;
      i=i+1;
   endo;
endif;

/* Plott all four solutions */

GraphSettings;
_pcolor=0;
_plwidth=7;
xlabel("Time");
ylabel("Capital Stock");
xtics(0,rows(kt1),1,0);
ymin=0.05;
ymax=0.22;
ytics(ymin,ymax,0.01,0);
_plegctl=(1~5~7~1.5*ymin);
_pltype=(6~1~6~3);
_plctrl={0 0 1 0};
_psymsiz=2;
_pstype=8;
_pcolor={0,0,0,0};
if eta /= 1;             
   _plegstr="Backward Iteration\000Forward Iteration\000Value Function Iteration";
   xy(seqa(1,1,rows(kt1)),kt3~kt1~kt2);
else;
   _plegstr="Backward Iteration\000Forward Iteration\000Value Function Iteration\000Analytic Solution";
   xt=((kt1-kt4)./kt4)~
      ((kt2-kt4)./kt4)~
      ((kt3-kt4)./kt4);      
   xy(seqa(1,1,rows(kt1)),kt1~kt2~kt3~kt4);
endif;      
output file=Ramsey2.txt reset;
datestring(date);
?"Run Time:";
?"Backward iteration:        " etstr(s1);
?"Forward iteration:         " etstr(s2);
?"Value function iteration1: " etstr(s3);
?"Value function iteration2: " etstr(s4);
if eta==1;
 ?"Maximum absolute percentage deviation from analytic solution at t";
 ?"Backward iteration:       " ftos(maxc(abs(xt[.,1]))*100,"*.*lf",10,8) maxindc(abs(xt[.,1]));
 ?"Forward iteration:        " ftos(maxc(abs(xt[.,2]))*100,"*.*lf",10,8) maxindc(abs(xt[.,2]));
 ?"Value function iteration: " ftos(maxc(abs(xt[.,3]))*100,"*.*lf",10,8) maxindc(abs(xt[.,3]));
endif;
output off;

ende:
end;
   

/* ----------------------------------- Subroutines ----------------------- */
   
/* Definition of the system of difference equations: x[1]=k and x[2]=c */

proc(1)=Sys1(x);

  local fx;
    
    fx=zeros(2,1);
    fx[1]=(x[1]^alpha) - x[2];
    fx[2]=x[2]*(alpha*beta*(((x[1]^alpha)-x[2])^(alpha-1)))^(1/eta);

 retp(fx);

endp;             

/* Sys2 defines the inverse of Sys1 */
  
proc(1)=Sys2(x); 

  local ct, kt;
  
  ct=x[2]*((alpha*beta*(x[1]^(alpha-1)))^(-1/eta));
  kt=(x[1]+ct)^(1/alpha);
  
retp(kt|ct);

endp;

/* The zero of Sys3 is the solution to the system of T equations in K_t */
proc(1)=Sys3(x);

   local f, i, nx;
   nx=rows(x);
   f=zeros(nx,1);
   f[1]=(((x[1]^alpha) - x[2])/((k0^alpha)-x[1]))-alpha*beta*(x[1]^(alpha-1));
      i=2;
      do until i>(nx-1);
          f[i]=((((x[i]^alpha) - x[i+1])/((x[i-1]^alpha)-x[i])))-alpha*beta*(x[i]^(alpha-1));
             i=i+1;
      endo;
      f[i]=((((x[i]^alpha) - ks)/((x[i-1]^alpha)-x[i])))-alpha*beta*(x[i]^(alpha-1));

 retp(f);

endp;


/* Definition of the utility function u=c^(1-eta)/(1-eta) as
** a function of this and next period's capital stock k1 and k2
** respectively */

proc(1)=u(k1,k2);

  local c;
  c=k1^alpha-k2;
  if c<0;
    c=miss(1,1);  @ set c to Gauss missing value code if c<0 @
  else;
    if eta==1;
      c=ln(c);
    else;  
      c=(c^(1-eta))/(1-eta);
    endif;
  endif;
 retp(c);
 
endp;
  
  
