@ ------------------------------------ Ramsey3a.g ----------------------------------

  Date:   08/07/2002
  Author: Alfred Maussner

  Purpose: Solve the stochastic infinite horizon Ramsey model with

            1. one-period utility function u(c)=c^(1-eta)/(1-eta) or
               u(C)=ln(c) for eta=1

            2. technology Z_t K_t^alpha

            3. transition equations

              K_{t+1} = Z_t K_t^alpha + (1-delta)K_t - C_t

              Z_{t} = Z_{t-1}^rho exp(epsilon_t)

              with epsiolon_t normally distributed with mean zero
              and variance sigma^2.

----------------------------------------------------------------------------------- @

new; @ clear memory @

library pgraph, dge, user;

/* Parameters of the problem */
alpha=0.35;               @ elasticity of production with respect to capital  @
beta=(1.065)^(-0.25);     @ dicont factor                                     @
eta=2;                    @ elasticity of marginal utility                    @
delta=0.01;               @ rate of capital depreciation                      @
sigma=0.01;               @ standard deviation of productivity shock          @
rho=0.95;                 @ autocorrelation coefficient of producticity shock @
T=150;                    @ Time considered necessary to approach the stationary equilibrium @
nobs=20;                  @ lenght of simulated time series                   @

/* stationary solution */
ks=((alpha*beta)/(1-beta*(1-delta)))^(1/(1-alpha));  @ stationary capital stock            @
cs=ks^alpha - delta*ks;                              @ stationary level of consumption     @

goto next1;  @ comment out if you want to compute the impulse response @

/* This part of the program computes the response of
** consumption and the stock of capital to a one-time productivity shock. */

bsec=hsec;

zt=zeros(t,1);
zt[1]=exp(sigma*rndn(1,1));   @ Draw a one-time shock from the standard normal @
zt[1]=exp(-sigma);            @ Used to derive the time paths in Figure 1.6    @
cls;
locate 2,2;
?"Size of shock= " zt[1] " Press any key to continue";wait;
i=1;
do until i>(t-1);             @ Compute the associated time path of the productivity shock @
     zt[i+1]=zt[i]^rho;
           i=i+1;
endo;

/* Solve for the time path of capital */
x0=ks*ones(T-1,1);               @ intitial values    @
k0=ks;                           @ set initial value of k equal to the stationary solution @
if zt[1]>1; kbar=(zt[1]/delta)^(1/(1-alpha)); else; kbar=delta^(1/(alpha-1)); endif; @ use maximal sustainable capital stock as upper bound for k @
bounds=zeros(T-1,1)~(ones(T-1,1).*kbar);
{fx,x1}=FixvMN(x0,bounds,&Sys1);   

/* Iterate over T: try a longer time path and check whether
** there is a noticable change between the shorter and the longer path */
x2=x1;
again:

t=t+5;
x0=x2|ones(5,1).*ks;
bounds=bounds|bounds[1:5,.];
i=1;
do until i>5;
   zt=zt|(zt[rows(zt)]^rho);
     i=i+1;
endo;

{fx,x1}=FixvMN(x0,bounds,&Sys1);
@krit=sqrt((x2-x1[1:rows(x2)])'(x2-x1[1:rows(x2)]));@
 krit=sqrt((x2[1:nobs]-x1[1:nobs])'(x2[1:nobs]-x1[1:nobs]));
if krit>1e-6;
locate 2,2;
?"t= " ftos(t,"*.*lf",5,0) " krit= " krit;
?"Press any key to continue...";wait;
   x2=x1;
   goto again;
endif;

kt=k0|x1[1:rows(x2)];

next:
kt=k0|x1;

/* Compute consumption */
i=1;
ct=zeros(rows(kt)-1,1);
do until i>rows(ct);
   ct[i]=ZT[i]*(KT[i]^alpha) + (1-delta)*kt[i]-kt[i+1];
       i=i+1;
endo;

s2=hsec-bsec;

/* Compute analytic solutions if eta=1 and delta=1 */
if (eta==1) and (delta==1);
  kt2=zeros(rows(kt),1);kt2[1]=ks;
  ct2=zeros(rows(kt)-1,1);       
  i=1;
  do until i>rows(kt2)-1;
     ct2[i]=(1-alpha*beta)*zt[i]*(kt2[i]^alpha);
   kt2[i+1]=alpha*beta*zt[i]*(kt2[i]^alpha);
          i=i+1;
  endo;
  xt=((kt-kt2)./kt2);      
  output file=Ramsey3a.out on;
  datestring(date) timestr(time);
  ?"Run Time:" etstr(s2);
  ?"Maximum absolute percentage deviation from analytic solution at t";
  ?"Forward iteration:        " ftos(maxc(abs(xt))*100,"*.*lf",10,8) maxindc(abs(xt));
  output off;
   
   
endif;
          
/* Plot the impulse response */
GraphSettings;
tend=minc(rows(zt)|rows(kt)|rows(ct));
begwind;
window(3,1,0);
xlabel("Time");
xtics(1,tend,1,0);
ylabel("Productivity Shock");
_plwidth=10;
_paxht=0.35;
_pnumht=0.30;
xy(seqa(1,1,tend),Zt[1:tend]~ones(tend,1));
nextwind;
ylabel("Capital Stock");
if (eta /= 1) or (delta /= 1);             
@   _plegstr="Extended Path\000Stationary Level";@
@   _plegctl=(1~7~5~minc(kt));@
   xy(seqa(1,1,tend),kt[1:tend]~ones(tend,1).*ks);
else;
@   _plegstr="Extended Path\000Analytic Solution\000Stationary Level";@
@   _plegctl=(1~7~5~minc(kt));@  
   xy(seqa(1,1,tend),kt[1:tend]~kt2[1:tend]~ones(tend,1).*ks);
endif;      
nextwind;
ylabel("Consumption");
if (eta==1) and (delta==1);
 xy(seqa(1,1,tend),ct[1:tend]~ct2[1:tend]~ones(tend,1).*cs);
else;
 xy(seqa(1,1,tend),ct[1:tend]~ones(tend,1).*cs);
endif;
endwind;

goto next2;

next1:

/* This part of the program simulates the model for a number of periods,
** t is set to 200, which has been found to give sufficient accuracy, even
** with large shocks */
bsec=hsec;
t=200;
eps=exp(sigma*rndn(nobs,1));  @ Derive the sequence of productivity shocks @

zshock=zeros(nobs,1);
zshock[1]=eps[1];
i=1;
do until i>(nobs-1);
  zshock[i+1]=(zshock[i]^rho)*eps[i+1];
            i=i+1;
endo;

kt=zeros(nobs+1,1);kt[1]=ks;  @ Start at stationary equilibrium @
ct=zeros(nobs,1);        
zbar=maxc(zshock);
kbar=(zbar/delta)^(1/(1-alpha));
bounds=zeros(T-1,1)~(ones(T-1,1).*kbar);
x0=ks*ones(T-1,1);               @ intitial values    @
itn=1;cls;

do while itn<=nobs;
  locate 5,5;?"Period #= " ftos(itn,"*.*lf",4,0);
  clear kex1, kex2, zex1, zex2;    @ auxilary variables @
  k0=kt[itn];                      @ set initial value of k equal to the previous stock of capital @
  zt=zeros(T,1);
  zt[1]=zshock[itn];               @ compute the expected time path of productivity shock from the current shock onward @
  i=1;
  do until i>(t-1);
     zt[i+1]=zt[i]^rho;
           i=i+1;
  endo;
  {fx,x1}=FixVMN(x0,bounds,&Sys1);
 
 kt[itn+1]=x1[1];
   ct[itn]=zshock[itn]*kt[itn]^alpha + (1-delta)*kt[itn] - kt[itn+1];
   x0=x1;        @ use old solution as guess for the next one @

itn=itn+1;

endo;
s3=hsec-bsec;

/* Compute the optimal path from the exact policy function, if eta==1 */
if (eta==1) and (delta==1);
   tend=rows(kt);
   kt2=zeros(tend,1);
   i=1;
   kt2[1]=kt[1];
   do until i>(tend-1);
      kt2[i+1]=alpha*beta*zshock[i]*kt2[i]^alpha;
      i=i+1;
   endo;
endif;

/* Plott the two solutions */

GraphSettings;
begwind;
window(3,1,0);
xlabel("Time");
xtics(1,nobs,1,0);
ylabel("Productivity Shock");
_plwidth=7;
_paxht=0.35;
_pnumht=0.30;
xy(seqa(1,1,nobs),ZShock~ones(nobs,1));
nextwind;
ylabel("Capital Stock");
if (eta /= 1) or (delta /= 1);             
@   _plegstr="Extended Path\000Stationary Level";@
@   _plegctl=(1~7~5~minc(kt));@
   xy(seqa(1,1,rows(kt)),kt~ones(rows(kt),1).*ks);
else;
@   _plegstr="Extended Path\000Analytic Solution\000Stationary Level";@
@   _plegctl=(1~7~5~minc(kt));@
   xt=((kt-kt2)./kt2);      
   xy(seqa(1,1,rows(kt)),kt~kt2~ones(rows(kt),1).*ks);
endif;      
nextwind;
ylabel("Consumption");
xy(seqa(1,1,nobs),ct~ones(rows(ct),1).*cs);
endwind;

output file=Ramsey3a.out on;
datestring(date);
?"Run Time:";
?"Forward iteration:         " etstr(s3);
if eta==1;
?"Maximum absolute percentage deviation from analytic solution at t";
?"Forward iteration:        " ftos(maxc(abs(xt))*100,"*.*lf",10,8) maxindc(abs(xt));
endif;
output off;

next2:
ende:
end;
   

/* ----------------------------------- Subroutines ----------------------- */
   

/* Sys1: Definition of the T-1 system of equations whose zero is
** the optimal sequence of capital stocks. */

proc(1)=Sys1(x0);

 external matrix zt, t, k0;
 local fx, i;
 
 fx=zeros(t-1,1);
 fx[1]=(((zt[2]*(x0[1]^alpha)+(1-delta)*x0[1] - x0[2])/
       (zt[1]*(k0^alpha)+(1-delta)*k0 - x0[1]))^eta) -
       beta*(1-delta+alpha*zt[2]*(x0[1]^(alpha-1)));
       
 i=2;
 do until i>(t-2);
  
   fx[i]=(((zt[i+1]*(x0[i]^alpha)+(1-delta)*x0[i] - x0[i+1])/
         (zt[i]*(x0[i-1]^alpha)+(1-delta)*x0[i-1] - x0[i]))^eta) -
         beta*(1-delta+alpha*zt[i+1]*(x0[i]^(alpha-1)));
       i=i+1;
 
 endo;
 
  fx[t-1]=(((zt[t]*(x0[t-1]^alpha)+(1-delta)*x0[t-1] - ks)/
          (zt[t-1]*(x0[t-2]^alpha)+(1-delta)*x0[t-2] - x0[t-1]))^eta) -
          beta*(1-delta+alpha*zt[t]*(x0[t-1]^(alpha-1)));
           
retp(fx);

endp;
