@ ------------------------------------ Ramsey3b.g ----------------------------------

  Date:    08/07/2002
  Author:  Alfred Maussner
  Purpose: Solve the stochastic infinite horizon Ramsey model with

                      1. one-period utility function
                      
                         u(c)=(c^(1-eta)/(1-eta))/(1-eta)
                         
                         or

                         u(C)=ln(c)
                         
                         for eta=1

                      2. technology Y = Z*K^alpha

                      3. transition equation

                         K' = Y + (1-delta)*K - C

                      4. Markov chain with elements z=[z1, z2, ..., zm]
                         and transition matrix p, where prob(zj|zi)=p[ij])

  Hint: if you run this program under Gauss 4.0, open the Dos compatibility
        window to ensure that the program's message print send to the
        screen are correctly displayed. 
                         

----------------------------------------------------------------------------------- @

new; @ clear memory @

library pgraph, dge, user;

/* Parameters of the problem */
alpha=0.27;                   @ elasticity of production with respect to capital    @
delta=0.01;                   @ rate of capital depreciation                        @
beta=(1.065)^(-0.25);         @ discount factor: 6.5% annual rate of return on capital@
eta=2;                        @ elasticity of marginal utility                    @
m=3;                          @ number of states                                  @

let z={0.99, 1.00, 1.01}; @ realizations of productivity shock                @
let p={0.61  0.34  0.05,
       0.24  0.52  0.24,
       0.05  0.34  0.61}; @ transition matrix                                 @


ngrid=1000;                @ number of grid points                             @
nobs=20;                   @ lenght of simulated time series                   @

zmax=maxc(z);
zmin=minc(z);



ksmax=((alpha*beta*zmax)/(1-beta*(1-delta)))^(1/(1-alpha));   @ maximum stationary capital stock @
ksmin=((alpha*beta*zmin)/(1-beta*(1-delta)))^(1/(1-alpha));   @ minimum stationary capital stock @
kstar=((alpha*beta     )/(1-beta*(1-delta)))^(1/(1-alpha));   

/* This part computes a grid over [k1,kn] where k1=0.95*the minimum stationary capital stock
** and kn=1.05*the maximum stationary capital stock.
*/
k1=0.95*ksmin;
kn=1.05*ksmax;
dk=(kn-k1)/(ngrid-1);

/* This part of the program computes the response of
** consumption and the stock of capital to a one-time productivity shock. */

bsec=hsec;
grid=seqa(k1,dk,ngrid);

ui1=u(z[1],ksmin,ksmin);
ui2=u(z[2],kstar,kstar);
ui3=u(z[3],ksmax,ksmax);
u0=(ones(ngrid,1).*ui1)~(ones(ngrid,1,).*ui2)~(ones(ngrid,1).*ui3);
v0=u0*inv(eye(cols(p))-beta*p');

g1=SolveVISb(&u,beta,grid,z,p,v0);


s1=hsec-bsec;
bsec=hsec;

output file=Ramsey3b.txt reset;
?datestring(0);
?timestr(0);
?"Run time to compute policy function= " etstr(s1);
output off;

/* Find the stationary solutions in g1 */
ks=GStat(g1);

goto next1; @ jump to the next1 section @

/* Compute impulse response: select an initial capital stock
** between ksmin and ksmax and initial, one time shock i1=1, 2, ..., cols(z)
** and the index of the shock that remains constant thereafter i2/=1, 2, ..., cols(z),
** i2/=i1 */


i2=2;
i1=1;
kt=ks[2];
kt=kt|g1[kt,i1];
j=kt[2];
do until g1[j,i2]==g1[g1[j,i2],i2];
   j=g1[j,i2];
   kt=kt|j;
endo;
kt=kt|g1[j,i2];

/* Compute time path from analytic solution if eta=1 and delta=1 */
if (eta==1) and (delta==1);
   kt2=zeros(rows(kt),1);
   kt2[1]=grid[kt[1]];
   kt2[2]=z[i1]*alpha*beta*(kt2[1]^alpha);
   i=2;
   do until i>(rows(kt2)-1);
      kt2[i+1]=alpha*beta*z[i2]*kt2[i]^alpha;
             i=i+1;
   endo;
   diff=maxc(abs(grid[kt]-kt2)./kt2)*100;
   output file=Ramsey3b.txt on;
     ?"Maximum Difference between analytic and approximate solution in percent: ";
     ?"diff= " ftos(diff,"*.*lf",15,10);
   output off;  
endif;   

GraphSettings;
if (eta==1) and (delta==1);
   xy(seqa(1,1,rows(kt)),grid[kt]~kt2~ones(rows(kt),1).*grid[ks[1]]~ones(rows(kt),1).*grid[ks[m]]);
else;  
@ xy(seqa(1,1,rows(kt)),grid[kt]~ones(rows(kt),1).*ks[1]~ones(rows(kt),1).*ks[m]);@
 xy(seqa(1,1,rows(kt)),grid[kt]);
endif;

next1:

/* This section computes a time path for the productivity shock
** and an associated time path for consumption and the capital stock.
*/

eps=rndu(nobs,1);  @ a sequence of uniformly distributed numbers from [0,1] @
zt=zeros(nobs+1,1);    @ zt stores the indeces of the shock                     @
zt[1]=2;             @ start with z=1                                         @
i=1;
m=rows(z);
do until i>nobs;   
   j=zt[i];
   if eps[i] <= p[j,1]; zt[i+1]=1; goto skip; endif;
   pu=p[j,1];po=pu;
   l=2;
   do until l>m;
      pu=po;
      po=pu+p[j,l];
      if ((eps[i] > pu) and (eps[i]<=po));  zt[i+1]=l; break; endif;       
      l=l+1;
   endo;
   skip:  
   i=i+1;
endo;      

/* Now compute time paths */
kt=zeros(nobs+1,1);

kt[1]=ks[2];  @ start at intermediate stationary equilibrium @
i=1;
do until i>nobs;
      kt[i+1]=g1[kt[i],zt[i]];
            i=i+1;
endo;
kt1=grid[kt];
zt1=z[zt];

/* Compute analytical solution */
if (eta==1) and (delta==1);
   kt2=zeros(nobs+1,1);
   kt2[1]=kt1[1];
   i=1;
   do until i>nobs;
      kt2[i+1]=alpha*beta*zt1[i]*(kt2[i]^alpha);
             i=i+1;
   endo;
   diff=maxc(abs(kt1-kt2)./kt2)*100;
   output file=Ramsey3b.txt on;
   ?"Maximum absolute difference between analytical and approximate solution in percent:";
   ?"diff= " ftos(diff,"*.*lf",15,10);
   output off;
endif;   
/* Compute consumption */
ct1=zeros(nobs,1);
i=1;
do until i>nobs;
   ct1[i]=zt1[i]*(kt1[i]^alpha)+(1-delta)*kt1[i]-kt1[i+1];
        i=i+1;
endo;
abhier:
cs=zeros(3,1);
cs[1]=z[1]*(grid[ks[1]]^alpha)+(1-delta)*grid[ks[1]]-grid[ks[1]];
cs[2]=z[2]*(grid[ks[2]]^alpha)+(1-delta)*grid[ks[2]]-grid[ks[2]];
cs[3]=z[3]*(grid[ks[3]]^alpha)+(1-delta)*grid[ks[3]]-grid[ks[3]];

GraphSettings;
_pcolor=0;
begwind;
window(3,1,0);
xlabel("Time");
xtics(1,nobs,1,0);
ylabel("Productivity Shock");
_plwidth=7;
_pcolor=0;
_pltype={6, 3, 3};
_paxht=0.35;
@xy(seqa(1,1,nobs),zt1[1:nobs]~ones(nobs,1).*zmin~ones(nobs,1).*zmax);@
xy(seqa(1,1,nobs),zt1[1:nobs]~ones(nobs,1).*z[2]);
nextwind;
ylabel("Capital Stock");@ytics(k1,kn,0.1,0);@
@xy(seqa(1,1,nobs),kt1[1:nobs]~ones(nobs,1).*grid[ks[1]]~ones(nobs,1).*grid[ks[3]]);@
xy(seqa(1,1,nobs),kt1[1:nobs]~ones(nobs,1).*grid[ks[2]]);
nextwind;
ylabel("Consumption");
@xy(seqa(1,1,nobs),ct1~ones(nobs,1).*cs[1]~ones(nobs,1).*cs[m]);@
xy(seqa(1,1,nobs),ct1~ones(nobs,1).*cs[2]);
endwind;

ende:
end;
   

/* ----------------------------------- Subroutines ----------------------- */
   
proc(1)=u(z,k1,k2);  @ defines the utility function @

  local c;
  c=z*(k1^alpha) + (1-delta)*k1 - k2;

  if c<0.0;
     c=miss(1,1);
  else;
     if eta==1;
       c=ln(c);
     else;
       c=(c^(1-eta))/(1-eta);
     endif;
  endif;

retp(c);

endp;

/* GStat: Find the stationary elements in the policy function g
**
** Usage: ks=GStat(g)
**
** Input:   g := n times m policy matrix
**
** Output: ks := m times 1 vector that holds the indices from g
**                 such that g[i,j]=i
*/

proc(1)=GStat(g);

   local ks, m, n, k, j;
   m=cols(g);
   ks=zeros(m,1);
   k=1;
   do until k>m;

   j=1;
   do until g[j,k]==g[g[j,k],k];
     j=g[j,k];  
   endo;
   ks[k]=g[j,k];
   k=k+1;
 
  endo;

retp(ks);

endp;
