@ ------------------------------------ Ramsey4b.g ----------------------------------

  Date:   08/12/2002
  Author: Alfred Maussner
  Purpose: Solve the stochastic infinite horizon Ramsey model with

                      1. one-period utility function
                      
                         u(c)=(c^(1-eta)/(1-eta))*((1-N)^(theta*(1-eta)))/(1-eta)
                         
                         or

                         u(C)=ln(c)+theta*ln(1-N)
                         
                         for eta=1

                      2. technology Y = Z*(AN^alpha)*( K^(1-alpha))

                      3. transition equations

                         A' = aA                        

                         K' = Y + (1-delta)*K - C

                      4. Markov chain with elements z=[z1, z2, ..., zm]
                         and transition matrix p, where prob(zj|zi)=p[ij]
                         approximating the process ln(Z')=rho*ln(Z) + eps
                         where eps normally distributed with mean zero
                         and standard deviation sigma

----------------------------------------------------------------------------------- @

new; @ clear memory @

library pgraph, dge, user;

/* Parameters of the problem */
a=1.005;                      @ gross growth rate of labor augmenting technical progress @
alpha=0.73;                   @ elasticity of production with respect to capital    @
delta=0.011;                  @ rate of capital depreciation                        @
eta=2;                        @ elasticity of marginal utility                      @
nstar=0.13;                   @ stationary fraction of hours worked                 @
rstar=1.0206;                 @ gross quarterly rate of return on stocks            @
rho=0.90;                     @ AR(1) parameter                                     @
sigma=0.0072;sigma=0.03;                 @ standard deviation of innovations                   @
nobs=60;                      @ length of simulated time series                     @
nofs=100;                     @ number of runs to compute average moments           @
lambda=1600;                  @ smoothing parameter of HP-Filter                    @

ngrid=500;                    @ number of grid points                             @
size=1.5;                     @ width of grid that approximates the AR(1) process @
m=9;                          @ number of possible states of Markov chain         @

/* Compute derived parameter values, stationary solution and grid */
beta=a^eta/rstar;
yk=((a^eta)-beta*(1-delta))/(beta*(1-alpha));   @ output-capital ratio      @
ck=yk+(1-delta-a);                              @ consumption-capital ratio @
yc=yk/ck;                                       @ inverse of average propensity to consume @
theta=(alpha*yc*(1-nstar))/nstar;               @ theta implied by nstar          @
kstar=(yk^(-1/alpha))*nstar;                    @ stationary capital stock k=K/A  @
istar=(a+delta-1)*kstar;                        @ stationary level of investment  @
cstar=yk*kstar - istar;                         @ stationary level of consumption @

{z,p}=MarkovAR(size,m, rho, sigma);            @ approximation of AR(1)          @

zmin=exp(minc(z));
zmax=exp(maxc(z));

kmax=nstar*(zmax^(1/alpha))*(yk^(-1/alpha));
kmin=nstar*(zmin^(1/alpha))*(yk^(-1/alpha));


/* This part computes a grid over [k1,kn] where k1=0.95*the minimum stationary capital stock
** and kn=1.05*the maximum stationary capital stock.
*/
k1=0.8*kmin;
kn=1.2*kmax;
dk=(kn-k1)/(ngrid-1);
grid=seqa(k1,dk,ngrid);

/* Compute the policy function */
clear z0, x1, x2;
v0=zeros(ngrid,m);
j=1;
do until j>m;
   i=1;
   kj=nstar*(yk/exp(z[j]))^(-1/alpha);
   do until i>ngrid;
       v0[i,j]=u(exp(z[j]),kj,kj);
      i=i+1;
   endo;
   j=j+1;
endo; 
v0=v0*inv(eye(cols(p))-beta*(a^(1-eta))*p');
bsec=hsec;
@g1=SolveVISa(&u,beta*(a^(1-eta)),grid,exp(z),p,v0);@
s1=hsec-bsec;
bsec=hsec;
g1=SolveVISb(&u,beta*(a^(1-eta)),grid,exp(z),p,v0);

/* Write Run Time to File */
s2=hsec-bsec;

output file=Ramsey4b1.txt reset;
?datestring(0);
?timestr(0);
?"Run time for Policy Function = " etstr(s2);

output off;

save g1;

/* Compute Impulse Respones for a one time shock */
goto next1;

i2=2;  @ Index of shock after time t=1 @
i1=3;  @ Index of shock at time t=1    @
j=1;   @ construct the optimal path from k0 onwards @

do until grid[j]>=kstar;
 j=j+1;
endo;

kt=j;
kt=kt|g1[j,i1];
j=kt[2];
do until g1[j,i2]==g1[g1[j,i2],i2];
   j=g1[j,i2];
   kt=kt|j;
endo;
kt=kt|g1[j,i2];

nobs1=rows(kt)-1;

zt=exp(z[i1]|(ones(nobs1-1,1).*z[i2]));
kt=grid[kt];
ct=zeros(nobs1,1);
nt=zeros(nobs1,1);

i=1;
do until i>nobs1;
   {ct[i],nt[i]}=GetCN(kt[i],kt[i+1],zt[i]);   
        i=i+1;
endo;


MyGraphSettings(0);
begwind;
window(2,2,0);
xlabel("Time");
xtics(1,nobs,1,0);
ylabel("Productivity Shock");
_plwidth=7;
_pcolor=0;
_pltype={6, 3, 3};
_paxht=0.35;
xy(seqa(1,1,nobs1),zt[1:nobs1]~ones(nobs1,1).*zmin~ones(nobs1,1).*zmax);
nextwind;
ylabel("Capital Stock");
xy(seqa(1,1,nobs1),kt[1:nobs1]~ones(nobs1,1).*kmin~ones(nobs1,1).*kmax);
nextwind;
ylabel("Consumption");
xy(seqa(1,1,nobs1),ct~ones(nobs1,1).*cstar);
nextwind;
ylabel("Hours");
xy(seqa(1,1,nobs1),nt~ones(nobs1,1).*nstar);
endwind;

goto ende;

/* This part simulates the model */
next1:
bsec=hsec;

ystar=(nstar^alpha)*(kstar^(1-alpha));
out1="Output  "~" "|
    "Investme"~"nt"|
    "Consumpt"~"ion"|
    "Hours"~"";
out2=zeros(4,3);
    
itn=1;
do until itn>nofs;

i0=floor((m/2))+1; @ about Z=1 @
zt=MarkovTP(nobs+1,p,i0);

/* Now compute time paths */
k=zeros(nobs+1,1);
i=1;
do while grid[i]<= kstar;
   i=i+1;
endo;
k[1]=i;
i=1;
do until i>nobs;
      k[i+1]=g1[k[i],zt[i]];
            i=i+1;
endo;
kt=grid[k];
zt=exp(z[zt]);

/* Compute output, investment, hours and consumption */
yt=zeros(nobs,1);
it=zeros(nobs,1);
ct=zeros(nobs,1);
nt=zeros(nobs,1);

i=1;
do until i>nobs;
   {ct[i],nt[i]}=GetCN(kt[i],kt[i+1],zt[i]);
   yt[i]=zt[i]*(nt[i]^alpha)*(kt[i]^(1-alpha));
    it[i]=a*kt[i+1]+(delta-1)*kt[i];
   
   yt[i]=(yt[i]-ystar)/ystar;
   it[i]=(it[i]-istar)/istar;
   ct[i]=(ct[i]-cstar)/cstar;
   nt[i]=(nt[i]-nstar)/nstar;
   
        i=i+1;
endo;

/* Filtering and computation of correlations */
yt=HPFilter(yt,1600);
it=HPFilter(it,1600);
ct=HPFilter(ct,1600);
nt=HPFilter(nt,1600);
x=yt~it~ct~nt;
cry=corrx(x);
std=stdc(x);
crx=corrx(x[1:nobs-1,.]~x[2:nobs,.]);
out2[.,1]=out2[.,1]+100*std;
out2[.,2]=out2[.,2]+cry[1,.]';
out2[.,3]=out2[.,3]+diag(crx[1:4,5:8]);

itn=itn+1;

endo;

out2=out2/nofs;

/* Write summary statistics to file */
out=out1~out2;
output on;
mask=(0~0~1~1~1);
fmt={"-*.*s" 8 8, "-*.*s" 8 8, "*.*lf" 8 2, "*.*lf" 8 2,"*.*lf" 8 2};

i=printfm(out,mask,fmt);
?"";?"";
?"Column 1: Variable Name";
?"Column 2: Standard Deviation";
?"Column 3: Cross Correlation with Ouput";
?"Column 4: First Order Autocorrelation";

?"Parameters:";
?"a     = " ftos(a,    "*.*lf",8,4);
?"alpha = " ftos(alpha,"*.*lf",8,4);
?"beta  = " ftos(beta, "*.*lf",8,4);
?"eta   = " ftos(eta,  "*.*lf",8,4);
?"nstar = " ftos(nstar,"*.*lf",8,4);
?"delta = " ftos(delta,"*.*lf",8,4);
?"rho   = " ftos(rho,  "*.*lf",8,4);
?"sigma = " ftos(sigma,"*.*lf",8,4);
?"size  = " ftos(size, "*.*lf",4,2);
?"m     = " ftos(m,    "*.*lf",2,0);
?"ngrid = " ftos(ngrid,"*.*lf",4,0);
?"nobs  = " ftos(nobs, "*.*lf",4,0);
?"nofs  = " ftos(nofs, "*.*lf",4,0);
?"";
?"Run Time Simulations =  " etstr(hsec-bsec);

output off;

ende:
end;
   

/* ----------------------------------- Subroutines ----------------------- */
   
proc(1)=u(z,k1,k2);  @ defines the utility function @

  local c, n, krit;
  external matrix z0, x1, x2;
  
  z0=z;x1=k1; x2=k2;
  
  @ Check whether there is a solution for n@
  krit=z*k1^(1-alpha)+(1-delta)*k1 - a*k2;
  if krit<0;
     c=miss(1,1);
  else;  
     n=FixpMN(nstar,(0~1),&GetN);
     c=z0*(n^alpha)*(x1^(1-alpha))+(1-delta)*x1-a*x2;      
     if c<0; 
        retp(miss(1,1)); 
     else;
       if eta==1;
         c=ln(c)+theta*ln(1-n);
       else;
         c=(c^(1-eta))*((1-n)^(theta*(1-eta)));
         c=c/(1-eta); 
      endif;  
     endif;
  endif;
  
retp(c);

endp;

proc(1)=GetN(n); @ defines the function whose zero is n @

 local fx;
 external matrix z0, x1, x2;
 
 fx=theta*(z0*(n^alpha)*(x1^(1-alpha))+(1-delta)*x1 - a*x2) -
    alpha*(1-n)*z0*(n^(alpha-1))*(x1^(1-alpha));
    
retp(fx);

endp; 


proc(2)=GetCN(k1,k2,z);

  local c, n;
  external matrix z0, x1, x2;
  
  z0=z;
  x1=k1;
  x2=k2;
  
  n=FixpMN(nstar,(0~1),&GetN);
  c=z0*(n^alpha)*(x1^(1-alpha))+(1-delta)*x1-a*x2;
  
retp(c,n);

endp;